require "sketchup"
require "ruby/rubydir"
require "singleton"

class AppObserver < Sketchup::AppObserver
  include Singleton
  @attached = false

  def onNewModel(*args)
    attach
    @q.each { |o| o.onNewModel(*args) }
  end

  def onOpenModel(*args)
    attach
    @q.each { |o| o.onOpenModel(args) }
  end

  def onQuit(*args)
    @q.each { |o| o.onQuit(args) }
  end

  def attach
    Sketchup.add_observer(self)
    @attached = true
  end
  private :attach

  def detach
    Sketchup.remove_observer(self)
    @attached = false
  end
  private :detach

  def register(o)
    attach unless @attached
    @q ||= []
    @q.push o unless @q.include? o
  end

  def unregister(o)
    @q.delete(o)
    detach if @q.empty?
  end

end # class AppObserver
